@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2004-2005 William Jon McCann <mccann@jhu.edu>
+ * Copyright (C) 2004-2006 William Jon McCann <mccann@jhu.edu>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -185,7 +185,7 @@ do_command (DBusConnection *connection)
                 DBusMessageIter iter;
                 dbus_bool_t     v;
 
-                reply = screensaver_send_message_void (connection, "getActive", TRUE);
+                reply = screensaver_send_message_void (connection, "GetActive", TRUE);
                 if (! reply) {
                         g_message ("Did not receive a reply from the screensaver.");
                         goto done;
@@ -207,11 +207,11 @@ do_command (DBusConnection *connection)
         }
 
         if (do_poke) {
-                reply = screensaver_send_message_void (connection, "Poke", FALSE);
+                reply = screensaver_send_message_void (connection, "SimulateUserActivity", FALSE);
         }
 
         if (do_activate) {
-                reply = screensaver_send_message_bool (connection, "setActive", TRUE);
+                reply = screensaver_send_message_bool (connection, "SetActive", TRUE);
                 if (! reply) {
                         g_message ("Did not receive a reply from the screensaver.");
                         goto done;
@@ -220,7 +220,7 @@ do_command (DBusConnection *connection)
         }
 
         if (do_deactivate) {
-                reply = screensaver_send_message_bool (connection, "setActive", FALSE);
+                reply = screensaver_send_message_bool (connection, "SetActive", FALSE);
                 if (! reply) {
                         g_message ("Did not receive a reply from the screensaver.");
                         goto done;
@@ -239,7 +239,7 @@ main (int    argc,
       char **argv)
 {
         DBusConnection *connection;
-        DBusError       dbus_error;        
+        DBusError       dbus_error;
         GOptionContext *context;
         gboolean        retval;
         GError         *error = NULL;
@@ -248,9 +248,9 @@ main (int    argc,
         bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 # ifdef HAVE_BIND_TEXTDOMAIN_CODESET
         bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-# endif 
+# endif
         textdomain (GETTEXT_PACKAGE);
-#endif 
+#endif
 
         g_type_init ();
 
