@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2004-2005 William Jon McCann <mccann@jhu.edu>
+ * Copyright (C) 2004-2006 William Jon McCann <mccann@jhu.edu>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -27,6 +27,7 @@
 #include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/types.h>		/* For uid_t, gid_t */
 
 #include <glib/gi18n.h>
 #include <gdk/gdkx.h>
@@ -42,6 +43,7 @@
 
 #include "file-transfer-dialog.h"
 
+#include "gs-visual-gl.h"
 #include "gs-job.h"
 #include "gs-prefs.h" /* for GS_MODE enum */
 
@@ -100,8 +102,9 @@ config_get_activate_delay (gboolean *is_writable)
 
         delay = gconf_client_get_int (client, KEY_ACTIVATE_DELAY, NULL);
 
-        if (delay < 1)
+        if (delay < 1) {
                 delay = 1;
+        }
 
         g_object_unref (client);
 
@@ -342,9 +345,12 @@ preview_clear (GtkWidget *widget)
 
 static void
 preview_set_theme (GtkWidget  *widget,
-                   const char *theme)
+                   const char *theme,
+                   const char *name)
 {
-        GError *error = NULL;
+        GError    *error = NULL;
+        GtkWidget *label;
+        char      *markup;
 
         if (job) {
                 gs_job_stop (job);
@@ -352,6 +358,11 @@ preview_set_theme (GtkWidget  *widget,
 
         preview_clear (widget);
 
+        label = glade_xml_get_widget (xml, "fullscreen_preview_theme_label");
+        markup = g_markup_printf_escaped ("<i>%s</i>", name);
+        gtk_label_set_markup (GTK_LABEL (label), markup);
+        g_free (markup);
+
         if ((theme && strcmp (theme, "__blank-only") == 0)) {
 
         } else if (theme && strcmp (theme, "__random") == 0) {
@@ -371,7 +382,6 @@ preview_set_theme (GtkWidget  *widget,
 
                         gs_job_start (job);
                 }
-                
         } else {
                 if (! gs_job_set_theme (job, theme, &error)) {
                         if (error) {
@@ -442,9 +452,10 @@ populate_model (GtkTreeStore *store)
 
         themes = get_theme_info_list ();
 
-        if (! themes)
+        if (themes == NULL) {
                 return;
-        
+        }
+
         for (l = themes; l; l = l->next) {
                 GSJobThemeInfo *info = l->data;
 
@@ -463,6 +474,39 @@ populate_model (GtkTreeStore *store)
         g_slist_free (themes);
 }
 
+static void
+tree_selection_previous (GtkTreeSelection *selection)
+{
+        GtkTreeIter   iter;
+        GtkTreeModel *model;
+        GtkTreePath  *path;
+
+        if (! gtk_tree_selection_get_selected (selection, &model, &iter)) {
+                return;
+        }
+
+        path = gtk_tree_model_get_path (model, &iter);
+        if (gtk_tree_path_prev (path)) {
+                gtk_tree_selection_select_path (selection, path);
+        }
+}
+
+static void
+tree_selection_next (GtkTreeSelection *selection)
+{
+        GtkTreeIter   iter;
+        GtkTreeModel *model;
+        GtkTreePath  *path;
+
+        if (! gtk_tree_selection_get_selected (selection, &model, &iter)) {
+                return;
+        }
+
+        path = gtk_tree_model_get_path (model, &iter);
+        gtk_tree_path_next (path);
+        gtk_tree_selection_select_path (selection, path);
+}
+
 static void
 tree_selection_changed_cb (GtkTreeSelection *selection,
                            GtkWidget        *preview)
@@ -470,19 +514,24 @@ tree_selection_changed_cb (GtkTreeSelection *selection,
         GtkTreeIter   iter;
         GtkTreeModel *model;
         char         *theme;
+        char         *name;
 
-        if (! gtk_tree_selection_get_selected (selection, &model, &iter))
+        if (! gtk_tree_selection_get_selected (selection, &model, &iter)) {
                 return;
+        }
 
-        gtk_tree_model_get (model, &iter, ID_COLUMN, &theme, -1);
+        gtk_tree_model_get (model, &iter, ID_COLUMN, &theme, NAME_COLUMN, &name, -1);
 
-        if (! theme)
+        if (! theme) {
+                g_free (name);
                 return;
+        }
 
-        preview_set_theme (preview, theme);
+        preview_set_theme (preview, theme, name);
         config_set_theme (theme);
 
         g_free (theme);
+        g_free (name);
 }
 
 static void
@@ -565,12 +614,13 @@ separator_func (GtkTreeModel *model,
 {
         int   column = GPOINTER_TO_INT (data);
         char *text;
-        
+
         gtk_tree_model_get (model, iter, column, &text, -1);
-        
-        if (text && strcmp (text, "__separator") == 0)
+
+        if (text != NULL && strcmp (text, "__separator") == 0) {
                 return TRUE;
-        
+        }
+
         g_free (text);
 
         return FALSE;
@@ -714,38 +764,26 @@ theme_installer_run (GtkWidget *parent,
 {
         GtkWidget    *dialog;
         GnomeVFSURI  *src_uri;
-        GList        *src;
-        GList        *target;
-        char         *user_dir;
-        char         *short_name;
-        char         *base_name;
+        GList        *src, *target;
         char         *target_path;
-        gboolean      is_desktop;
+        char         *user_dir;
+        char         *base;
 
         src_uri = gnome_vfs_uri_new (filename);
         src = g_list_append (NULL, src_uri);
-        target = NULL;
-        target_path = NULL;
 
         user_dir = g_build_filename (g_get_user_data_dir (), "gnome-screensaver", "themes", NULL);
-        g_mkdir_with_parents (user_dir, S_IRWXU);
+        base = gnome_vfs_uri_extract_short_name (src_uri);
 
-        short_name = gnome_vfs_uri_extract_short_name (src_uri);
-        base_name = NULL;
-
-        is_desktop = FALSE;
-        if (short_name != NULL
-            && g_str_has_suffix (short_name, ".desktop")) {
-                /* FIXME: validate key file? */
-                is_desktop = TRUE;
-                base_name = g_strndup (short_name, strlen (short_name) - 8);
-        }
+        target_path = NULL;
 
         while (TRUE) {
-                char *tmp;
-                gboolean exists;
+                char      *file_tmp;
+                int        len = strlen (base);
 
-                if (! is_desktop) {
+                if (base && len > 4 && (!strcmp (base + len - 4, ".xml"))) {
+                        file_tmp = g_strdup_printf ("screensaver-theme-%d.xml", rand ());
+                } else {
                         dialog = gtk_message_dialog_new (GTK_WINDOW (parent),
                                                          GTK_DIALOG_MODAL,
                                                          GTK_MESSAGE_ERROR,
@@ -760,25 +798,20 @@ theme_installer_run (GtkWidget *parent,
                         gtk_dialog_run (GTK_DIALOG (dialog));
                         gtk_widget_destroy (dialog);
                         g_free (target_path);
-                        goto out;
+                        return;
                 }
 
-                g_free (target_path);
-                target_path = g_build_filename (user_dir, short_name, NULL);
+                target_path = g_build_filename (user_dir, file_tmp, NULL);
 
-                /* FIXME: racy, oh well */
-                exists = gnome_vfs_uri_exists (gnome_vfs_uri_new (target_path));
-                if (! exists) {
-                        target = g_list_append (NULL, gnome_vfs_uri_new (target_path));
+                g_free (file_tmp);
+                if (! gnome_vfs_uri_exists (gnome_vfs_uri_new (target_path)))
                         break;
-                }
-
-                /* try another name */
-                tmp = g_strdup_printf ("%s-%u.desktop",  base_name, g_random_int ());
-                g_free (short_name);
-                short_name = tmp;
         }
 
+        g_free (user_dir);
+
+        target = g_list_append (NULL, gnome_vfs_uri_new (target_path));
+
         dialog = file_transfer_dialog_new ();
         gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (parent));
         gtk_window_set_icon_name (GTK_WINDOW (dialog), "screensaver");
@@ -789,6 +822,8 @@ theme_installer_run (GtkWidget *parent,
                                               GNOME_VFS_XFER_ERROR_MODE_QUERY,
                                               GNOME_VFS_XFER_OVERWRITE_MODE_QUERY,
                                               GNOME_VFS_PRIORITY_DEFAULT);
+        gnome_vfs_uri_list_unref (src);
+        gnome_vfs_uri_list_unref (target);
 
         g_signal_connect (dialog, "cancel",
                           G_CALLBACK (transfer_cancel_cb), target_path);
@@ -796,14 +831,6 @@ theme_installer_run (GtkWidget *parent,
                           G_CALLBACK (transfer_done_cb), target_path);
 
         gtk_widget_show (dialog);
-
- out:
-        gnome_vfs_uri_list_unref (src);
-        gnome_vfs_uri_list_unref (target);
-
-        g_free (user_dir);
-        g_free (base_name);
-        g_free (short_name);
 }
 
 /* Callback issued during drag movements */
@@ -850,13 +877,14 @@ drag_data_received_cb (GtkWidget        *widget,
         if (uris != NULL && uris->data != NULL) {
                 GnomeVFSURI *uri = (GnomeVFSURI *) uris->data;
 
-                if (gnome_vfs_uri_is_local (uri)) {
-                        filename = gnome_vfs_unescape_string (gnome_vfs_uri_get_path (uri),
-                                                              G_DIR_SEPARATOR_S);
-                } else {
-                        filename = gnome_vfs_unescape_string (gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_NONE),
-                                                              G_DIR_SEPARATOR_S);
-                }
+                if (gnome_vfs_uri_is_local (uri))
+                        filename = gnome_vfs_unescape_string (
+                                        gnome_vfs_uri_get_path (uri),
+                                        G_DIR_SEPARATOR_S);
+                else
+                        filename = gnome_vfs_unescape_string (
+                                        gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_NONE),
+                                        G_DIR_SEPARATOR_S);
 
                 gnome_vfs_uri_list_unref (uris);
         }
@@ -1054,17 +1082,165 @@ key_changed_cb (GConfClient *client,
         }
 }
 
+static void
+fullscreen_preview_previous_cb (GtkWidget *fullscreen_preview_window,
+                                gpointer   user_data)
+{
+        GtkWidget        *treeview;
+        GtkTreeSelection *selection;
+
+        treeview = glade_xml_get_widget (xml, "savers_treeview");
+        selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
+        tree_selection_previous (selection);
+}
+
+static void
+fullscreen_preview_next_cb (GtkWidget *fullscreen_preview_window,
+                            gpointer   user_data)
+{
+        GtkWidget        *treeview;
+        GtkTreeSelection *selection;
+
+        treeview = glade_xml_get_widget (xml, "savers_treeview");
+        selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
+        tree_selection_next (selection);
+}
+
+static void
+fullscreen_preview_cancelled_cb (GtkWidget *button,
+                                 gpointer   user_data)
+{
+
+        GtkWidget *fullscreen_preview_area;
+        GtkWidget *fullscreen_preview_window;
+        GtkWidget *preview_area;
+        GtkWidget *dialog;
+
+        preview_area = glade_xml_get_widget (xml, "preview_area");
+        gs_job_set_widget (job, preview_area);
+
+        fullscreen_preview_area = glade_xml_get_widget (xml, "fullscreen_preview_area");
+        preview_clear (fullscreen_preview_area);
+
+        fullscreen_preview_window = glade_xml_get_widget (xml, "fullscreen_preview_window");
+        gtk_widget_hide (fullscreen_preview_window);
+
+        dialog = glade_xml_get_widget (xml, "prefs_dialog");
+        gtk_widget_show (dialog);
+        gtk_window_present (GTK_WINDOW (dialog));
+}
+
+static void
+fullscreen_preview_start_cb (GtkWidget *widget,
+                             gpointer   user_data)
+{
+        GtkWidget *fullscreen_preview_area;
+        GtkWidget *fullscreen_preview_window;
+        GtkWidget *dialog;
+
+        dialog = glade_xml_get_widget (xml, "prefs_dialog");
+        gtk_widget_hide (dialog);
+
+        fullscreen_preview_window = glade_xml_get_widget (xml, "fullscreen_preview_window");
+        gtk_widget_show (fullscreen_preview_window);
+        gtk_widget_grab_focus (fullscreen_preview_window);
+
+        gtk_window_fullscreen (GTK_WINDOW (fullscreen_preview_window));
+        gtk_window_set_keep_above (GTK_WINDOW (fullscreen_preview_window), TRUE);
+
+        fullscreen_preview_area = glade_xml_get_widget (xml, "fullscreen_preview_area");
+        gs_job_set_widget (job, fullscreen_preview_area);
+}
+
+static void
+constrain_list_size (GtkWidget      *widget,
+                     GtkRequisition *requisition,
+                     GtkWidget      *to_size)
+{
+        GtkRequisition req;
+        int            max_height;
+
+        /* constrain height to be the tree height up to a max */
+        max_height = (gdk_screen_get_height (gtk_widget_get_screen (widget))) / 4;
+
+        gtk_widget_size_request (to_size, &req);
+
+        requisition->height = MIN (req.height, max_height);
+}
+
+static void
+setup_list_size_constraint (GtkWidget *widget,
+                            GtkWidget *to_size)
+{
+        g_signal_connect (widget, "size-request",
+                          G_CALLBACK (constrain_list_size), to_size);
+}
+
+static gboolean
+check_is_root_user (void)
+{
+#ifndef G_OS_WIN32
+  uid_t ruid, euid, suid; /* Real, effective and saved user ID's */
+  gid_t rgid, egid, sgid; /* Real, effective and saved group ID's */
+
+#ifdef HAVE_GETRESUID
+  /* These aren't in the header files, so we prototype them here.
+   */
+  int getresuid(uid_t *ruid, uid_t *euid, uid_t *suid);
+  int getresgid(gid_t *rgid, gid_t *egid, gid_t *sgid);
+
+  if (getresuid (&ruid, &euid, &suid) != 0 ||
+      getresgid (&rgid, &egid, &sgid) != 0)
+#endif /* HAVE_GETRESUID */
+    {
+      suid = ruid = getuid ();
+      sgid = rgid = getgid ();
+      euid = geteuid ();
+      egid = getegid ();
+    }
+
+  if (ruid == 0) {
+          return TRUE;
+  }
+
+#endif
+  return FALSE;
+}
+
+static void
+setup_for_root_user (void)
+{
+        GtkWidget *lock_checkbox;
+        GtkWidget *label;
+
+        lock_checkbox = glade_xml_get_widget (xml, "lock_checkbox");
+        label = glade_xml_get_widget (xml, "root_warning_label");
+
+        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lock_checkbox), FALSE);
+        gtk_widget_set_sensitive (lock_checkbox, FALSE);
+
+        gtk_widget_show (label);
+}
+
 static void
 init_capplet (void)
 {
         GtkWidget *dialog;
         GtkWidget *preview;
         GtkWidget *treeview;
+        GtkWidget *list_scroller;
         GtkWidget *activate_delay_hscale;
         GtkWidget *activate_delay_hbox;
         GtkWidget *label;
         GtkWidget *enabled_checkbox;
         GtkWidget *lock_checkbox;
+        GtkWidget *root_warning_label;
+        GtkWidget *preview_button;
+        GtkWidget *fullscreen_preview_window;
+        GtkWidget *fullscreen_preview_previous;
+        GtkWidget *fullscreen_preview_next;
+        GtkWidget *fullscreen_preview_area;
+        GtkWidget *fullscreen_preview_close;
         char      *glade_file;
         char      *string;
         gdouble    activate_delay;
@@ -1095,39 +1271,54 @@ init_capplet (void)
         preview            = glade_xml_get_widget (xml, "preview_area");
         dialog             = glade_xml_get_widget (xml, "prefs_dialog");
         treeview           = glade_xml_get_widget (xml, "savers_treeview");
+        list_scroller      = glade_xml_get_widget (xml, "themes_scrolled_window");
         activate_delay_hscale = glade_xml_get_widget (xml, "activate_delay_hscale");
         activate_delay_hbox   = glade_xml_get_widget (xml, "activate_delay_hbox");
         enabled_checkbox   = glade_xml_get_widget (xml, "enable_checkbox");
         lock_checkbox      = glade_xml_get_widget (xml, "lock_checkbox");
+        root_warning_label = glade_xml_get_widget (xml, "root_warning_label");
+        preview_button     = glade_xml_get_widget (xml, "preview_button");
+        fullscreen_preview_window = glade_xml_get_widget (xml, "fullscreen_preview_window");
+        fullscreen_preview_area = glade_xml_get_widget (xml, "fullscreen_preview_area");
+        fullscreen_preview_close = glade_xml_get_widget (xml, "fullscreen_preview_close");
+        fullscreen_preview_previous = glade_xml_get_widget (xml, "fullscreen_preview_previous_button");
+        fullscreen_preview_next = glade_xml_get_widget (xml, "fullscreen_preview_next_button");
 
         label              = glade_xml_get_widget (xml, "activate_delay_label");
         gtk_label_set_mnemonic_widget (GTK_LABEL (label), activate_delay_hscale);
         label              = glade_xml_get_widget (xml, "savers_label");
         gtk_label_set_mnemonic_widget (GTK_LABEL (label), treeview);
 
+        gtk_widget_set_no_show_all (root_warning_label, TRUE);
+        gs_visual_gl_widget_set_best_colormap (preview);
+
         activate_delay = config_get_activate_delay (&is_writable);
         ui_set_delay (activate_delay);
-        if (! is_writable)
+        if (! is_writable) {
                 gtk_widget_set_sensitive (activate_delay_hbox, FALSE);
+        }
         g_signal_connect (activate_delay_hscale, "format-value",
                           G_CALLBACK (format_value_callback_time), NULL);
 
         gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lock_checkbox), config_get_lock (&is_writable));
-        if (! is_writable)
+        if (! is_writable) {
                 gtk_widget_set_sensitive (lock_checkbox, FALSE);
+        }
         g_signal_connect (lock_checkbox, "toggled",
                           G_CALLBACK (lock_checkbox_toggled), NULL);
 
         enabled = config_get_enabled (&is_writable);
         ui_set_enabled (enabled);
-        if (! is_writable)
+        if (! is_writable) {
                 gtk_widget_set_sensitive (enabled_checkbox, FALSE);
+        }
         g_signal_connect (enabled_checkbox, "toggled",
                           G_CALLBACK (enabled_checkbox_toggled), NULL);
 
-            
+        setup_list_size_constraint (list_scroller, treeview);
+        gtk_widget_set_size_request (preview, 480, 300);
         gtk_window_set_icon_name (GTK_WINDOW (dialog), "screensaver");
-
+        gtk_window_set_icon_name (GTK_WINDOW (fullscreen_preview_window), "screensaver");
 
         gtk_drag_dest_set (dialog, GTK_DEST_DEFAULT_ALL,
                            drop_types, G_N_ELEMENTS (drop_types),
@@ -1145,7 +1336,7 @@ init_capplet (void)
         /* Update list of themes if using random screensaver */
         client = gconf_client_get_default ();
         string = gconf_client_get_string (client, KEY_MODE, NULL);
-        if (string) {
+        if (string != NULL) {
                 int mode;
                 GSList *list;
 
@@ -1177,11 +1368,26 @@ init_capplet (void)
         setup_treeview (treeview, preview);
         setup_treeview_selection (treeview);
 
+        if (check_is_root_user ()) {
+                setup_for_root_user ();
+        }
+
         g_signal_connect (activate_delay_hscale, "value-changed",
                           G_CALLBACK (activate_delay_value_changed_cb), NULL);
 
         g_signal_connect (dialog, "response",
                           G_CALLBACK (response_cb), NULL);
+
+        g_signal_connect (preview_button, "clicked",
+                          G_CALLBACK (fullscreen_preview_start_cb),
+                          treeview);
+
+        g_signal_connect (fullscreen_preview_close, "clicked", 
+                          G_CALLBACK (fullscreen_preview_cancelled_cb), NULL);
+        g_signal_connect (fullscreen_preview_previous, "clicked", 
+                          G_CALLBACK (fullscreen_preview_previous_cb), NULL);
+        g_signal_connect (fullscreen_preview_next, "clicked", 
+                          G_CALLBACK (fullscreen_preview_next_cb), NULL);
 }
 
 int
@@ -1193,9 +1399,9 @@ main (int    argc,
         bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 # ifdef HAVE_BIND_TEXTDOMAIN_CODESET
         bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-# endif 
+# endif
         textdomain (GETTEXT_PACKAGE);
-#endif 
+#endif
 
         gnome_program_init (PACKAGE, VERSION,
                             LIBGNOMEUI_MODULE,
