@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2004-2005 William Jon McCann <mccann@jhu.edu>
+ * Copyright (C) 2004-2006 William Jon McCann <mccann@jhu.edu>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,9 +20,6 @@
  *
  */
 
-/* DPMS support has been moved to gnome-power-manager */
-#undef USE_LEGACY_DPMS_SUPPORT
-
 #include "config.h"
 #include <stdlib.h>
 #include <stdio.h>
@@ -37,10 +34,7 @@
 #include "gs-manager.h"
 #include "gs-watcher.h"
 #include "gs-fade.h"
-
-#ifdef USE_LEGACY_DPMS_SUPPORT
-#include "gs-power.h"
-#endif
+#include "gs-grab.h"
 
 #include "gs-listener-dbus.h"
 #include "gs-monitor.h"
@@ -58,11 +52,11 @@ struct GSMonitorPrivate
         GSWatcher      *watcher;
         GSListener     *listener;
         GSManager      *manager;
-#ifdef USE_LEGACY_DPMS_SUPPORT
-        GSPower        *power;
-#endif
         GSPrefs        *prefs;
         GSFade         *fade;
+        GSGrab         *grab;
+
+        guint           release_grab_id;
 };
 
 enum {
@@ -100,6 +94,20 @@ manager_deactivated_cb (GSManager *manager,
         gs_listener_set_active (monitor->priv->listener, FALSE);
 }
 
+static void
+manager_auth_request_begin_cb (GSManager *manager,
+                               GSMonitor *monitor)
+{
+        gs_listener_emit_auth_request_begin (monitor->priv->listener);
+}
+
+static void
+manager_auth_request_end_cb (GSManager *manager,
+                             GSMonitor *monitor)
+{
+        gs_listener_emit_auth_request_end (monitor->priv->listener);
+}
+
 static gboolean
 watcher_idle_cb (GSWatcher *watcher,
                  gboolean   is_idle,
@@ -114,6 +122,20 @@ watcher_idle_cb (GSWatcher *watcher,
         return res;
 }
 
+static gboolean
+release_grab_timeout (GSMonitor *monitor)
+{
+        gboolean manager_active;
+
+        manager_active = gs_manager_get_active (monitor->priv->manager);
+        if (! manager_active) {
+                gs_grab_release (monitor->priv->grab);
+        }
+
+        monitor->priv->release_grab_id = 0;
+        return FALSE;
+}
+
 static gboolean
 watcher_idle_notice_cb (GSWatcher *watcher,
                         gboolean   in_effect,
@@ -133,8 +155,12 @@ watcher_idle_notice_cb (GSWatcher *watcher,
         if (in_effect) {
                 if (activation_enabled && ! inhibited) {
                         /* start slow fade */
-                        gs_fade_set_timeout (monitor->priv->fade, FADE_TIMEOUT);
-                        gs_fade_set_active (monitor->priv->fade, TRUE);
+                        if (gs_grab_grab_offscreen (monitor->priv->grab, FALSE)) {
+                                gs_fade_async (monitor->priv->fade, FADE_TIMEOUT, NULL, NULL);
+                        } else {
+                                gs_debug ("Could not grab the keyboard so not performing idle warning fade-out");
+                        }
+
                         handled = TRUE;
                 }
         } else {
@@ -143,8 +169,14 @@ watcher_idle_notice_cb (GSWatcher *watcher,
                 manager_active = gs_manager_get_active (monitor->priv->manager);
                 /* cancel the fade unless manager was activated */
                 if (! manager_active) {
-                        gs_fade_set_active (monitor->priv->fade, FALSE);
+                        gs_debug ("manager not active, performing fade cancellation");
                         gs_fade_reset (monitor->priv->fade);
+
+                        /* don't release the grab immediately to prevent typing passwords into windows */
+                        if (monitor->priv->release_grab_id != 0) {
+                                g_source_remove (monitor->priv->release_grab_id);
+                        }
+                        monitor->priv->release_grab_id = g_timeout_add (1000, (GSourceFunc)release_grab_timeout, monitor);
                 } else {
                         gs_debug ("manager active, skipping fade cancellation");
                 }
@@ -160,11 +192,17 @@ listener_lock_cb (GSListener *listener,
                   GSMonitor  *monitor)
 {
         gboolean res;
+        gboolean locked;
 
+        /* set lock flag before trying to activate screensaver
+           in case something tries to react to the ActiveChanged signal */
+
+        gs_manager_get_lock_active (monitor->priv->manager, &locked);
+        gs_manager_set_lock_active (monitor->priv->manager, TRUE);
         res = gs_listener_set_active (monitor->priv->listener, TRUE);
-        if (res) {
-                gs_manager_set_lock_active (monitor->priv->manager, TRUE);
-        } else {
+        if (! res) {
+                /* If we've failed then restore lock status */
+                gs_manager_set_lock_active (monitor->priv->manager, locked);
                 gs_debug ("Unable to lock the screen");
         }
 }
@@ -200,17 +238,6 @@ listener_active_changed_cb (GSListener *listener,
                 goto done;
         }
 
-#ifdef USE_LEGACY_DPMS_SUPPORT
-                /* enable power management */
-                res = gs_power_set_active (monitor->priv->power, active);
-                if (! res) {
-                        gs_debug ("Unable to set power management active: %d", active);
-
-                        /* if we can't activate power management it isn't the
-                           end of the world */
-                }
-#endif
-
         ret = TRUE;
 
  done:
@@ -235,45 +262,18 @@ listener_throttle_changed_cb (GSListener *listener,
 }
 
 static void
-listener_poke_cb (GSListener *listener,
-                  GSMonitor  *monitor)
+listener_simulate_user_activity_cb (GSListener *listener,
+                                    GSMonitor  *monitor)
 {
         /* in case the screen isn't blanked reset the
            idle watcher */
         gs_watcher_reset (monitor->priv->watcher);
 
-#ifdef USE_LEGACY_DPMS_SUPPORT
-        /* turn on the monitor power */
-        gs_power_set_mode (monitor->priv->power,
-                           GS_POWER_MODE_ON);
-#endif
-
         /* request that the manager unlock -
            will pop up a dialog if necessary */
         gs_manager_request_unlock (monitor->priv->manager);
 }
 
-#ifdef USE_LEGACY_DPMS_SUPPORT
-static void
-power_changed_cb (GSPower    *power,
-                  GSPowerMode mode,
-                  GSMonitor  *monitor)
-{
-        gboolean is_on;
-
-        if (mode == GS_POWER_MODE_ON) {
-                is_on = TRUE;
-        } else {
-                is_on = FALSE;
-        }
-
-        /* Don't run themes if the monitor power is off */
-        if (! is_on) {
-                gs_manager_set_throttled (monitor->priv->manager, TRUE);
-        }
-}
-#endif
-
 static void
 _gs_monitor_update_from_prefs (GSMonitor *monitor,
                                GSPrefs   *prefs)
@@ -314,16 +314,6 @@ _gs_monitor_update_from_prefs (GSMonitor *monitor,
         if (activate_watch) {
                 gs_watcher_set_active (monitor->priv->watcher, TRUE);
         }
-
-
-#ifdef USE_LEGACY_DPMS_SUPPORT
-        gs_power_set_timeouts (monitor->priv->power,
-                               monitor->priv->prefs->dpms_standby,
-                               monitor->priv->prefs->dpms_suspend,
-                               monitor->priv->prefs->dpms_off);
-        gs_power_set_enabled (monitor->priv->power,
-                              monitor->priv->prefs->dpms_enabled);
-#endif
 }
 
 static void
@@ -334,7 +324,7 @@ disconnect_listener_signals (GSMonitor *monitor)
         g_signal_handlers_disconnect_by_func (monitor->priv->listener, listener_cycle_cb, monitor);
         g_signal_handlers_disconnect_by_func (monitor->priv->listener, listener_active_changed_cb, monitor);
         g_signal_handlers_disconnect_by_func (monitor->priv->listener, listener_throttle_changed_cb, monitor);
-        g_signal_handlers_disconnect_by_func (monitor->priv->listener, listener_poke_cb, monitor);
+        g_signal_handlers_disconnect_by_func (monitor->priv->listener, listener_simulate_user_activity_cb, monitor);
 }
 
 static void
@@ -350,8 +340,8 @@ connect_listener_signals (GSMonitor *monitor)
                           G_CALLBACK (listener_active_changed_cb), monitor);
         g_signal_connect (monitor->priv->listener, "throttle-changed",
                           G_CALLBACK (listener_throttle_changed_cb), monitor);
-        g_signal_connect (monitor->priv->listener, "poke",
-                          G_CALLBACK (listener_poke_cb), monitor);
+        g_signal_connect (monitor->priv->listener, "simulate-user-activity",
+                          G_CALLBACK (listener_simulate_user_activity_cb), monitor);
 }
 
 static void
@@ -375,6 +365,8 @@ disconnect_manager_signals (GSMonitor *monitor)
 {
         g_signal_handlers_disconnect_by_func (monitor->priv->manager, manager_activated_cb, monitor);
         g_signal_handlers_disconnect_by_func (monitor->priv->manager, manager_deactivated_cb, monitor);
+        g_signal_handlers_disconnect_by_func (monitor->priv->manager, manager_auth_request_begin_cb, monitor);
+        g_signal_handlers_disconnect_by_func (monitor->priv->manager, manager_auth_request_end_cb, monitor);
 }
 
 static void
@@ -384,23 +376,12 @@ connect_manager_signals (GSMonitor *monitor)
                           G_CALLBACK (manager_activated_cb), monitor);
         g_signal_connect (monitor->priv->manager, "deactivated",
                           G_CALLBACK (manager_deactivated_cb), monitor);
+        g_signal_connect (monitor->priv->manager, "auth-request-begin",
+                          G_CALLBACK (manager_auth_request_begin_cb), monitor);
+        g_signal_connect (monitor->priv->manager, "auth-request-end",
+                          G_CALLBACK (manager_auth_request_end_cb), monitor);
 }
 
-#ifdef USE_LEGACY_DPMS_SUPPORT
-static void
-disconnect_power_signals (GSMonitor *monitor)
-{
-        g_signal_handlers_disconnect_by_func (monitor->priv->power, power_changed_cb, monitor);
-}
-
-static void
-connect_power_signals (GSMonitor *monitor)
-{
-        g_signal_connect (monitor->priv->power, "changed",
-                          G_CALLBACK (power_changed_cb), monitor);
-}
-#endif
-
 static void
 disconnect_prefs_signals (GSMonitor *monitor)
 {
@@ -427,17 +408,14 @@ gs_monitor_init (GSMonitor *monitor)
         connect_listener_signals (monitor);
 
         monitor->priv->fade = gs_fade_new ();
+        monitor->priv->grab = gs_grab_new ();
+
         monitor->priv->watcher = gs_watcher_new (monitor->priv->prefs->timeout);
         connect_watcher_signals (monitor);
 
         monitor->priv->manager = gs_manager_new ();
         connect_manager_signals (monitor);
 
-#ifdef USE_LEGACY_DPMS_SUPPORT
-        monitor->priv->power = gs_power_new ();
-        connect_power_signals (monitor);
-#endif
-
         _gs_monitor_update_from_prefs (monitor, monitor->priv->prefs);
 }
 
@@ -456,18 +434,13 @@ gs_monitor_finalize (GObject *object)
         disconnect_watcher_signals (monitor);
         disconnect_listener_signals (monitor);
         disconnect_manager_signals (monitor);
-#ifdef USE_LEGACY_DPMS_SUPPORT
-        disconnect_power_signals (monitor);
-#endif
         disconnect_prefs_signals (monitor);
 
         g_object_unref (monitor->priv->fade);
+        g_object_unref (monitor->priv->grab);
         g_object_unref (monitor->priv->watcher);
         g_object_unref (monitor->priv->listener);
         g_object_unref (monitor->priv->manager);
-#ifdef USE_LEGACY_DPMS_SUPPORT
-        g_object_unref (monitor->priv->power);
-#endif
         g_object_unref (monitor->priv->prefs);
 
         G_OBJECT_CLASS (parent_class)->finalize (object);
